use super::song::Song;
use rand::seq::SliceRandom;
use std::{cmp::min, collections::VecDeque};

pub struct SongQueue {
    queue: VecDeque<Song>,
}

impl SongQueue {
    pub fn new() -> SongQueue {
        SongQueue {
            queue: VecDeque::new(),
        }
    }
    pub async fn push(&mut self, songs: Vec<Song>) {
        for item in songs.into_iter() {
            self.queue.push_back(item);
        }
    }
    pub async fn pop(&mut self) -> Option<Song> {
        self.queue.pop_front()
    }
    pub async fn shuffle(&mut self) -> Result<(), String> {
        if self.queue.len() == 0 {
            return Err("queue is empty".to_string());
        }
        self.queue
            .make_contiguous()
            .shuffle(&mut rand::thread_rng());

        Ok(())
    }
    pub async fn clear(&mut self) -> Result<(), String> {
        if self.queue.len() == 0 {
            return Err("queue is empty".to_string());
        };
        self.queue.clear();
        Ok(())
    }
    pub async fn get_string(&self) -> String {
        if self.queue.len() == 0 {
            return "*empty*".to_string();
        };
        let mut s = String::new();
        s.push_str(&format!(
            "*Showing {} of {} songs*\n",
            min(20, self.queue.len()),
            self.queue.len()
        ));
        for (i, song) in self.queue.iter().take(20).enumerate() {
            s += &format!("{}: ", i);
            s += &song.get_string().await;
            s += "\n";
        }
        s
    }
}
